@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: ephy-find-toolbar.c,v 1.20 2005/11/08 12:56:23 chpe Exp $
+ *  $Id: ephy-find-toolbar.c,v 1.26 2006/01/10 12:50:56 paharvey Exp $
  */
 
 #include "config.h"
@@ -40,8 +40,6 @@
 #include <gtk/gtkmain.h>
 #include <string.h>
 
-#undef FIND_WHILE_TYPING_IN_EMBED
-
 #define EPHY_FIND_TOOLBAR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object),EPHY_TYPE_FIND_TOOLBAR, EphyFindToolbarPrivate))
 
 struct _EphyFindToolbarPrivate
@@ -50,6 +48,7 @@ struct _EphyFindToolbarPrivate
 	EphyWindow *window;
 	EphyEmbed *embed;
 	GtkWidget *entry;
+	GtkWidget *label;
 	GtkToolItem *next;
 	GtkToolItem *prev;
 	GtkToolItem *sep;
@@ -59,7 +58,6 @@ struct _EphyFindToolbarPrivate
 	guint preedit_changed : 1;
 	guint prevent_activate : 1;
 	guint activated : 1;
-	guint explicit_focus : 1;
 	guint links_only : 1;
 	guint typing_ahead : 1;
 };
@@ -127,10 +125,10 @@ set_status (EphyFindToolbar *toolbar,
 			text = NULL;
 			break;
 		case EPHY_EMBED_FIND_NOTFOUND:
-			text = _("Phrase not found");
+			text = _("Not found");
 			break;
 		case EPHY_EMBED_FIND_FOUNDWRAPPED:
-			text = _("Find wrapped");
+			text = _("Wrapped");
 			break;
 	}
 
@@ -148,6 +146,9 @@ clear_status (EphyFindToolbar *toolbar)
 
 	gtk_widget_hide (GTK_WIDGET (priv->sep));
 	gtk_widget_hide (GTK_WIDGET (priv->status_item));
+	gtk_label_set_text (GTK_LABEL (priv->status_label), "");
+	gtk_label_set_text (GTK_LABEL (priv->label),
+			    priv->links_only ? _("Find links:") : _("Find:"));
 }
 
 static void
@@ -160,42 +161,6 @@ tab_content_changed_cb (EphyEmbed *embed,
 
 #ifdef HAVE_TYPEAHEADFIND
 
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-
-/* Cut and paste from gtkwindow.c */
-static void
-send_focus_change (GtkWidget *widget,
-		   gboolean   in)
-{
-	GdkEvent *event;
-
-	event = gdk_event_new (GDK_FOCUS_CHANGE);
-
-	g_object_ref (widget);
-   
-	if (in)
-	{
-		GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-	}
-	else
-	{
-		GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-	}
-
-	event->focus_change.type = GDK_FOCUS_CHANGE;
-	event->focus_change.window = (GdkWindow *) g_object_ref (widget->window);
-	event->focus_change.in = in;
-  
-	gtk_widget_event (widget, event);
-  
-	g_object_notify (G_OBJECT (widget), "has_focus");
-
-	g_object_unref (widget);
-	gdk_event_free (event);
-}
-
-#endif /* FIND_WHILE_TYPING_IN_EMBED */
-
 /* Code adapted from gtktreeview.c:gtk_tree_view_key_press() and
  * gtk_tree_view_real_start_interactive_seach()
  */
@@ -206,12 +171,6 @@ tab_search_key_press_cb (EphyEmbed *embed,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	GtkWidget *widget = (GtkWidget *) toolbar;
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-	GtkEntry *entry = (GtkEntry *) priv->entry;
-	GdkWindow *event_window;
-	gboolean retval = FALSE;
-	guint oldhash, newhash;
-#endif
 
 	g_return_val_if_fail (event != NULL, FALSE);
 
@@ -234,58 +193,8 @@ tab_search_key_press_cb (EphyEmbed *embed,
 	}
 
 	return FALSE;
-
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-	/* don't do anything if the find toolbar is hidden */
-	if (GTK_WIDGET_VISIBLE (widget) == FALSE ||
-	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter ||
-	    event->keyval == GDK_ISO_Enter)
-	{
-		return FALSE;
-	}
-
-	oldhash = g_str_hash (gtk_entry_get_text (entry));
-
-	event_window = event->window;
-	event->window = priv->entry->window;
-
-	/* Send the event to the window.  If the preedit_changed signal is emitted
-	* during this event, we will set priv->imcontext_changed  */
-	priv->preedit_changed = priv->activated = FALSE;
-	priv->prevent_activate = TRUE;
-	retval = gtk_widget_event (priv->entry, (GdkEvent*) event);
-	priv->prevent_activate = FALSE;
-
-	/* restore event window, else gdk_event_free below will crash */
-	event->window = event_window;
-
-	newhash = g_str_hash (gtk_entry_get_text (entry));
-
-	/* FIXME: is this correct? */
-	return retval && (oldhash != newhash || priv->preedit_changed) && !priv->activated;
-#endif /* FIND_WHILE_TYPING_IN_EMBED */
 }
 
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-
-static gboolean
-tab_dom_mouse_click_cb (EphyEmbed *embed,
-			gpointer event,
-			EphyFindToolbar *toolbar)
-{
-	EphyFindToolbarPrivate *priv = toolbar->priv;
-
-	if (!priv->explicit_focus)
-	{
-		ephy_find_toolbar_close (toolbar);
-	}
-
-	return FALSE;
-}
-
-#endif /* FIND_WHILE_TYPING_IN_EMBED */
-
 #endif /* HAVE_TYPEAHEADFIND */
 
 static void
@@ -307,7 +216,9 @@ entry_changed_cb (GtkEntry *entry,
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	const char *text;
 	char *lowercase;
+#ifdef HAVE_TYPEAHEADFIND
 	EphyEmbedFindResult result;
+#endif
 	gboolean found = TRUE, case_sensitive;
 
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
@@ -323,37 +234,55 @@ entry_changed_cb (GtkEntry *entry,
 #ifdef HAVE_TYPEAHEADFIND
 	result = ephy_embed_find_find (get_find (toolbar), text, priv->links_only);
 
-	found = result == EPHY_EMBED_FIND_FOUND;
+	found = result != EPHY_EMBED_FIND_NOTFOUND;
 	set_status (toolbar, result);
 #endif
 	set_controls (toolbar, found, found);
 }
 
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-static void
-entry_preedit_changed_cb (GtkIMContext *context,
-			  EphyFindToolbar *toolbar)
-{
-	toolbar->priv->preedit_changed = TRUE;
-}
-#endif
-
 static gboolean
 entry_key_press_event_cb (GtkEntry *entry,
 			  GdkEventKey *event,
 			  EphyFindToolbar *toolbar)
 {
+	EphyFindToolbarPrivate *priv = toolbar->priv;
 	guint mask = gtk_accelerator_get_default_mod_mask ();
 	gboolean handled = FALSE;
 
-	/* Hide the toolbar when ESC is pressed */
-	if ((event->state & mask) == 0 && event->keyval == GDK_Escape)
+	if ((event->state & mask) == 0)
 	{
-		g_signal_emit (toolbar, signals[CLOSE], 0);
 		handled = TRUE;
+		switch (event->keyval)
+		{
+		case GDK_Up:
+                case GDK_KP_Up:
+			ephy_embed_scroll (priv->embed, -1);
+			break;
+		case GDK_Down:
+                case GDK_KP_Down:
+			ephy_embed_scroll (priv->embed, 1);
+			break;
+		case GDK_Page_Up:
+                case GDK_KP_Page_Up:
+			ephy_embed_page_scroll (priv->embed, -1);
+			break;
+		case GDK_Page_Down:
+                case GDK_KP_Page_Down:
+			ephy_embed_page_scroll (priv->embed, 1);
+			break;
+		case GDK_Escape:
+			/* Hide the toolbar when ESC is pressed */
+			ephy_find_toolbar_request_close (toolbar);
+			break;
+		default:
+			handled = FALSE;
+			break;
+		}
 	}
 	else if ((event->state & mask) == GDK_CONTROL_MASK &&
-		 (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter))
+		 (event->keyval == GDK_Return ||
+		  event->keyval == GDK_KP_Enter ||
+		  event->keyval == GDK_ISO_Enter))
 	{
 		handled = ephy_embed_find_activate_link (get_find (toolbar), event->state);
 	}
@@ -367,11 +296,6 @@ entry_activate_cb (GtkWidget *entry,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-	priv->activated = TRUE;
-	if (priv->prevent_activate) return;
-#endif
-
 	if (priv->typing_ahead)
 	{
 		ephy_embed_find_activate_link (get_find (toolbar), 0);
@@ -387,7 +311,6 @@ set_focus_cb (EphyWindow *window,
 	      GtkWidget *widget,
 	      EphyFindToolbar *toolbar)
 {
-	EphyFindToolbarPrivate *priv = toolbar->priv;
 	GtkWidget *wtoolbar = GTK_WIDGET (toolbar);
 
 	while (widget != NULL && widget != wtoolbar)
@@ -396,24 +319,9 @@ set_focus_cb (EphyWindow *window,
 	}
 
 	/* if widget == toolbar, the new focus widget is in the toolbar */
-	if (widget == wtoolbar)
+	if (widget != wtoolbar)
 	{
-		priv->explicit_focus = TRUE;
-	}
-	else if (priv->explicit_focus)
-	{
-		g_signal_emit (toolbar, signals[CLOSE], 0);
-	}
-}
-
-static void
-sync_print_preview_mode (EphyWindow *window,
-			 GParamSpec *pspec,
-			 EphyFindToolbar *toolbar)
-{
-	if (ephy_window_get_is_print_preview (window) && GTK_WIDGET_VISIBLE (GTK_WIDGET (toolbar)))
-	{
-		ephy_find_toolbar_close (toolbar);
+		ephy_find_toolbar_request_close (toolbar);
 	}
 }
 
@@ -424,10 +332,6 @@ ephy_find_toolbar_set_window (EphyFindToolbar *toolbar,
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
 	priv->window = window;
-
-	sync_print_preview_mode (window, NULL, toolbar);
-	g_signal_connect (window, "notify::print-preview-mode",
-			  G_CALLBACK (sync_print_preview_mode), toolbar);
 }
 
 static void
@@ -454,7 +358,6 @@ ephy_find_toolbar_grab_focus (GtkWidget *widget)
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
 	gtk_widget_grab_focus (GTK_WIDGET (priv->entry));
-	g_return_if_fail (priv->explicit_focus);
 }
 
 static void
@@ -463,7 +366,7 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	EphyFindToolbarPrivate *priv;
 	GtkToolbar *gtoolbar;
 	GtkToolItem *item;
-	GtkWidget *alignment, *arrow, *box, *label;
+	GtkWidget *alignment, *arrow, *box;
 
 	priv = toolbar->priv = EPHY_FIND_TOOLBAR_GET_PRIVATE (toolbar);
 	gtoolbar = GTK_TOOLBAR (toolbar);
@@ -477,9 +380,8 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	box = gtk_hbox_new (FALSE, 12);
 	gtk_container_add (GTK_CONTAINER (alignment), box);
 
-	label = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (label), _("Find:"));
-	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
+	priv->label = gtk_label_new (NULL);
+	gtk_box_pack_start (GTK_BOX (box), priv->label, FALSE, FALSE, 0);
 
 	priv->entry = gtk_entry_new ();
 	gtk_entry_set_width_chars (GTK_ENTRY (priv->entry), 32);
@@ -494,7 +396,6 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 
 	/* Prev */
 	arrow = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
-	label = gtk_label_new (_("Find Previous"));
 	priv->prev = gtk_tool_button_new (arrow, _("Find Previous"));
 	gtk_tool_item_set_is_important (priv->prev, TRUE);
 	gtk_tool_item_set_tooltip (priv->prev, gtoolbar->tooltips,
@@ -505,7 +406,6 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 
 	/* Next */
 	arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
-	label = gtk_label_new (_("Find Next"));
 	priv->next = gtk_tool_button_new (arrow, _("Find Next"));
 	gtk_tool_item_set_is_important (priv->next, TRUE);
 	gtk_tool_item_set_tooltip (priv->next, gtoolbar->tooltips,
@@ -531,10 +431,6 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 			  G_CALLBACK (entry_key_press_event_cb), toolbar);
 	g_signal_connect_after (priv->entry, "changed",
 				G_CALLBACK (entry_changed_cb), toolbar);
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-	g_signal_connect (GTK_ENTRY (priv->entry)->im_context, "preedit-changed",
-			  G_CALLBACK (entry_preedit_changed_cb), toolbar);
-#endif
 	g_signal_connect (priv->entry, "activate",
 			  G_CALLBACK (entry_activate_cb), toolbar);
 	g_signal_connect_swapped (priv->next, "clicked",
@@ -600,7 +496,6 @@ ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 
 	klass->next = ephy_find_toolbar_find_next;
 	klass->previous = ephy_find_toolbar_find_previous;
-	klass->close = ephy_find_toolbar_close;
 
 	signals[NEXT] =
 		g_signal_new ("next",
@@ -705,6 +600,8 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 	if (embed != NULL)
 	{
 		set_controls (toolbar, TRUE, TRUE);
+		clear_status (toolbar);
+
 		g_signal_connect_object (embed, "ge-content-change",
 					 G_CALLBACK (tab_content_changed_cb),
 					 toolbar, G_CONNECT_AFTER);
@@ -712,11 +609,6 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 		g_signal_connect_object (embed, "ge-search-key-press",
 					 G_CALLBACK (tab_search_key_press_cb),
 					 toolbar, 0);
-#ifdef FIND_WHILE_TYPING_IN_EMBED
-		g_signal_connect_object (embed, "dom-mouse-click",
-					 G_CALLBACK (tab_dom_mouse_click_cb),
-					 toolbar, 0);
-#endif /* FIND_WHILE_TYPING_IN_EMBED */
 #endif /* HAVE_TYPEAHEADFIND */
 
 		if (priv->find != NULL)
@@ -736,7 +628,7 @@ ephy_find_toolbar_find_next (EphyFindToolbar *toolbar)
 
 	result = ephy_embed_find_find_again (get_find (toolbar), TRUE);
 
-	found = result == EPHY_EMBED_FIND_FOUND;
+	found = result != EPHY_EMBED_FIND_NOTFOUND;
 	set_controls (toolbar, found, found);
 	set_status (toolbar, result);
 }
@@ -748,7 +640,7 @@ ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
 	gboolean found;
 
 	result = ephy_embed_find_find_again (get_find (toolbar), FALSE);
-	found = result == EPHY_EMBED_FIND_FOUND;
+	found = result != EPHY_EMBED_FIND_NOTFOUND;
 	set_controls (toolbar, found, found);
 	set_status (toolbar, result);
 }
@@ -765,7 +657,6 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
 	priv->typing_ahead = typing_ahead;
 	priv->links_only = links_only;
-	priv->explicit_focus = FALSE;
 
 	clear_status (toolbar);
 
@@ -780,19 +671,7 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
 	gtk_widget_show (GTK_WIDGET (toolbar));
 
-#if defined(HAVE_TYPEAHEADFIND) && defined(FIND_WHILE_TYPING_IN_EMBED)
-	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
-
-	send_focus_change (priv->entry, TRUE);
-
-	if (strlen(gtk_entry_get_text (GTK_ENTRY (priv->entry))) != 0)
-	{
-		entry_changed_cb (GTK_ENTRY (priv->entry), toolbar);
-	}
-#else
 	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
-#endif
-
 }
 
 void
@@ -800,13 +679,17 @@ ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	g_return_if_fail (priv->embed != NULL);
-	
 	gtk_widget_hide (GTK_WIDGET (toolbar));
 
-	/* first unset explicit_focus, else we get infinite recursion */
-	priv->explicit_focus = FALSE;
-#ifndef FIND_WHILE_TYPING_IN_EMBED
-	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
-#endif
+	if (priv->embed == NULL || priv->find == NULL) return;
+	ephy_embed_find_set_selection (get_find (toolbar), FALSE);
+}
+
+void
+ephy_find_toolbar_request_close (EphyFindToolbar *toolbar)
+{
+	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (toolbar)))
+	{
+		g_signal_emit (toolbar, signals[CLOSE], 0);
+	}
 }
